'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _getJobKey = require('./get-job-key');

var _getJobKey2 = _interopRequireDefault(_getJobKey);

var _getQueryCost = require('./get-query-cost');

var _getQueryCost2 = _interopRequireDefault(_getQueryCost);

var _isInTree = require('./is-in-tree');

var _isInTree2 = _interopRequireDefault(_isInTree);

var _isPluralParam = require('./is-plural-param');

var _isPluralParam2 = _interopRequireDefault(_isPluralParam);

var _pathKeyToRouteQueryKey = require('./path-key-to-route-query-key');

var _pathKeyToRouteQueryKey2 = _interopRequireDefault(_pathKeyToRouteQueryKey);

var _pathsToTree = require('./paths-to-tree');

var _pathsToTree2 = _interopRequireDefault(_pathsToTree);

var _pathToRoute = require('./path-to-route');

var _pathToRoute2 = _interopRequireDefault(_pathToRoute);

var _queryToPaths = require('./query-to-paths');

var _queryToPaths2 = _interopRequireDefault(_queryToPaths);

var _resolvePath = require('./resolve-path');

var _resolvePath2 = _interopRequireDefault(_resolvePath);

var _routeToParams = require('./route-to-params');

var _routeToParams2 = _interopRequireDefault(_routeToParams);

var _store = require('./store');

var _store2 = _interopRequireDefault(_store);

var _syncPromise = require('./sync-promise');

var _syncPromise2 = _interopRequireDefault(_syncPromise);

var _toKey = require('./to-key');

var _toKey2 = _interopRequireDefault(_toKey);

var _treeToQuery = require('./tree-to-query');

var _treeToQuery2 = _interopRequireDefault(_treeToQuery);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var partitionPaths = function partitionPaths(paths) {
  var mutations = [];
  var reads = [];
  for (var i = 0, l = paths.length; i < l; ++i) {
    var path = paths[i];
    var key = path[0];
    if (typeof key === 'string' && key[key.length - 1] === '!') {
      mutations.push(path);
    } else {
      reads.push(path);
    }
  }
  return { mutations: mutations, reads: reads };
};

var Router = function () {
  function Router() {
    var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var maxQueryCost = _ref.maxQueryCost;
    var _ref$routes = _ref.routes;
    var routes = _ref$routes === undefined ? {} : _ref$routes;

    _classCallCheck(this, Router);

    this.maxQueryCost = maxQueryCost;
    this.routes = routes;
  }

  _createClass(Router, [{
    key: 'getRouteForPath',
    value: function getRouteForPath(path) {
      var routes = this.routes;

      var query = [];
      for (var i = 0, l = path.length; i < l; ++i) {
        query[i] = (0, _pathKeyToRouteQueryKey2.default)(path[i]);
      }

      for (var _i = query.length; _i > 0; --_i) {
        var paths = (0, _queryToPaths2.default)(query.slice(0, _i));
        for (var j = 0, _l = paths.length; j < _l; ++j) {
          var route = (0, _pathToRoute2.default)(paths[j]);
          var fn = routes[route];
          if (fn) return { route: route, fn: fn };
        }
      }

      throw new Error('No route matches ' + JSON.stringify(path));
    }
  }, {
    key: 'run',
    value: function run(_ref2) {
      var _this = this;

      var _ref2$query = _ref2.query;
      var query = _ref2$query === undefined ? [] : _ref2$query;
      var force = _ref2.force;
      var _ref2$deltas = _ref2.deltas;
      var deltas = _ref2$deltas === undefined ? [] : _ref2$deltas;
      var _ref2$store = _ref2.store;
      var store = _ref2$store === undefined ? new _store2.default() : _ref2$store;
      var onlyUnresolved = _ref2.onlyUnresolved;

      return _syncPromise2.default.resolve().then(function () {
        var limit = _this.maxQueryCost;

        if (limit && (0, _getQueryCost2.default)(query, limit) > limit) {
          throw Router.EXPENSIVE_QUERY_ERROR;
        }

        var paths = (0, _queryToPaths2.default)(query);
        if (force !== true) {
          var forceTree = (0, _pathsToTree2.default)(force ? (0, _queryToPaths2.default)(force) : []);
          var undefPaths = [];
          for (var i = 0, l = paths.length; i < l; ++i) {
            var path = paths[i];
            var resolved = (0, _resolvePath2.default)(store.cache, path);
            if (onlyUnresolved && path === resolved) continue;

            var isUndef = store.get(resolved) === undefined;
            if (isUndef || (0, _isInTree2.default)(path, forceTree)) undefPaths.push(resolved);
          }
          paths = undefPaths;
        }

        if (!paths.length) return deltas;

        var _partitionPaths = partitionPaths(paths);

        var mutations = _partitionPaths.mutations;
        var reads = _partitionPaths.reads;

        if (mutations.length) {
          paths = mutations;
          query = [reads];
          onlyUnresolved = false;
        } else {
          query = [paths];
          force = false;
          onlyUnresolved = true;
        }

        var jobs = {};
        for (var _i2 = 0, _l2 = paths.length; _i2 < _l2; ++_i2) {
          var _path = paths[_i2];

          var _getRouteForPath = _this.getRouteForPath(_path);

          var route = _getRouteForPath.route;
          var fn = _getRouteForPath.fn;

          var params = (0, _routeToParams2.default)(route);
          var jobKey = (0, _getJobKey2.default)(params, _path);
          var job = jobs[jobKey];
          if (!job) job = jobs[jobKey] = { fn: fn, paths: [], args: { store: store }, keys: {} };

          job.paths.push(_path);

          var _job = job;
          var keys = _job.keys;
          var args = _job.args;

          for (var _i3 = 0; _i3 < params.length; ++_i3) {
            var param = params[_i3];
            var arg = _path[_i3];
            if ((0, _isPluralParam2.default)(param)) {
              if (!args[_i3]) {
                args[_i3] = [];
                keys[_i3] = {};
              }
              var key = (0, _toKey2.default)(arg);
              if (keys[_i3][key]) continue;
              args[_i3].push(arg);
              keys[_i3][key] = true;
            } else args[_i3] = arg;
          }
        }

        var work = [];
        for (var _key in jobs) {
          var _jobs$_key = jobs[_key];
          var fn = _jobs$_key.fn;
          var _paths = _jobs$_key.paths;
          var args = _jobs$_key.args;

          args.query = (0, _treeToQuery2.default)((0, _pathsToTree2.default)(_paths));
          work.push(_syncPromise2.default.resolve(args).then(fn).then(function (newDeltas) {
            store.update(newDeltas);
            deltas.push(newDeltas);
          }));
        }

        return _syncPromise2.default.all(work).then(function () {
          return _this.run({ query: query, force: force, deltas: deltas, store: store, onlyUnresolved: onlyUnresolved });
        });
      });
    }
  }]);

  return Router;
}();

Router.EXPENSIVE_QUERY_ERROR = new Error('Query is too expensive');
exports.default = Router;
