'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _applyDelta = require('./apply-delta');

var _applyDelta2 = _interopRequireDefault(_applyDelta);

var _deferred = require('./deferred');

var _deferred2 = _interopRequireDefault(_deferred);

var _get2 = require('./get');

var _get3 = _interopRequireDefault(_get2);

var _getRaw2 = require('./get-raw');

var _getRaw3 = _interopRequireDefault(_getRaw2);

var _queryToPaths = require('./query-to-paths');

var _queryToPaths2 = _interopRequireDefault(_queryToPaths);

var _resolvePath = require('./resolve-path');

var _resolvePath2 = _interopRequireDefault(_resolvePath);

var _router = require('./router');

var _router2 = _interopRequireDefault(_router);

var _triggerWatchers = require('./trigger-watchers');

var _triggerWatchers2 = _interopRequireDefault(_triggerWatchers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var runBatch = function runBatch(store) {
  var _store$batch = store.batch;
  var _store$batch$deferred = _store$batch.deferred;
  var resolve = _store$batch$deferred.resolve;
  var reject = _store$batch$deferred.reject;
  var force = _store$batch.force;
  var query = _store$batch.query;

  delete store.batch;
  store.router.run({ query: query, force: force, store: store }).then(resolve, reject);
};

var Store = function () {
  function Store() {
    var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var _ref$batchDelay = _ref.batchDelay;
    var batchDelay = _ref$batchDelay === undefined ? 0 : _ref$batchDelay;
    var _ref$cache = _ref.cache;
    var cache = _ref$cache === undefined ? {} : _ref$cache;
    var _ref$router = _ref.router;
    var router = _ref$router === undefined ? new _router2.default() : _ref$router;

    _classCallCheck(this, Store);

    this.batchDelay = batchDelay;
    this.cache = cache;
    this.router = router;
    this.watchers = [];
  }

  _createClass(Store, [{
    key: 'get',
    value: function get(path) {
      return (0, _get3.default)(this.cache, path);
    }
  }, {
    key: 'getRaw',
    value: function getRaw(path) {
      return (0, _getRaw3.default)(this.cache, path);
    }
  }, {
    key: 'resolve',
    value: function resolve(path) {
      return (0, _resolvePath2.default)(this.cache, path);
    }
  }, {
    key: 'update',
    value: function update(delta) {
      var next = (0, _applyDelta2.default)(this.cache, delta);
      var prev = this.cache;

      if (next === prev) return this;

      this.cache = next;
      (0, _triggerWatchers2.default)(this.watchers, prev, next);
      return this;
    }
  }, {
    key: 'run',
    value: function run(_ref2) {
      var query = _ref2.query;
      var force = _ref2.force;
      var batchDelay = this.batchDelay;

      if (!batchDelay) return this.router.run({ query: query, force: force, store: this });

      if (!this.batch) {
        this.batch = {
          deferred: new _deferred2.default(),
          force: [[]],
          query: [[]],
          timeoutId: setTimeout(runBatch, batchDelay, this)
        };
      }

      this.batch.query[0].push(query);
      if (force) this.batch.force[0].push(force === true ? query : force);
      return this.batch.deferred.promise;
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      var _batch = this.batch;
      _batch = _batch === undefined ? {} : _batch;
      var timeoutId = _batch.timeoutId;

      if (timeoutId) clearTimeout(timeoutId);
    }
  }, {
    key: 'watch',
    value: function watch(query, cb) {
      this.unwatch(cb).watchers.push({ paths: (0, _queryToPaths2.default)(query), cb: cb });
      return this;
    }
  }, {
    key: 'unwatch',
    value: function unwatch(cb) {
      var watchers = this.watchers;


      if (!cb) watchers.length = 0;

      for (var i = watchers.length - 1; i >= 0; --i) {
        if (watchers[i].cb === cb) watchers.splice(i, 1);
      }

      return this;
    }
  }]);

  return Store;
}();

exports.default = Store;
